
/* Check to see if this file has been included already.  */
#ifndef         AG101_REGS
#define         AG101_REGS

/* Different memory size defines used by SDRAM initialization */

#define    SDRAM_256MB            0x0100
#define    SDRAM_128MB            0x00C0
#define    SDRAM_64MB             0x0040
#define    SDRAM_32MB             0x0004
#define    SDRAM_16MB             0x0000

#define TIMER_BASE             0x98400000  /* Define base for all timer registers */
#define TIMER1_CNT_OFFSET      0x00        /* Offset for timer1 counter register */
#define TIMER1_LOAD_OFFSET     0x04        /* Offset for timer1 load register */
#define TIMER1_MATCH1_OFFSET   0x08        /* Offset for timer1 counter */
#define TIMER1_MATCH2_OFFSET   0x0c        /* Offset for timer1 counter */
#define TIMER2_CNT_OFFSET      0x10        /* Offset for timer2 counter register */
#define TIMER2_LOAD_OFFSET     0x14        /* Offset for timer2 load register */
#define TIMER2_MATCH1_OFFSET   0x18        /* Offset for timer2 counter */
#define TIMER2_MATCH2_OFFSET   0x1c        /* Offset for timer2 counter */
#define TIMER3_CNT_OFFSET      0x20        /* Offset for timer3 counter register */
#define TIMER3_LOAD_OFFSET     0x24        /* Offset for timer3 load register */
#define TIMER3_MATCH1_OFFSET   0x28        /* Offset for timer3 counter */
#define TIMER3_MATCH2_OFFSET   0x2c        /* Offset for timer3 counter */
#define TIMER_CNTRL_OFFSET     0x30        /* Offset for timer control register */
#define TIMER_INTS_OFFSET      0x34        /* Offset for timer interrupt status register */
#define TIMER_INTM_OFFSET      0x38        /* Offset for timer interrupt mask register */

#define INTC_TIMER1_MASK        0x00080000  /* Mask to enable timer interrupt */
#define INTC_TIMER1_CLEAR_VALUE 0x00080000  /* Value to clear pending timer1 interrupt */
#define INTC_TIMER_CLEAR_VALUE  0x0008c000  /* Value to clear all pending timer interrupt */
#define INTC_TIMER1_TRIGGER     0x00080000  /* bit 19  = 1: Edge trigger */
#define INTC_TIMER1_LEVEL       0x00000000  /* bit 19  = 0: Falling edge Trigger */

#define TIMER1_INTM_INIT        0x00000003  /* Mask out timer1 match interrupts */
#define TIMER_CNTRL_DISABLE     0x00000000  /* Value to disable timer */
#define TIMER1_CTRL_INIT        0x00000005  /* Bit 2   = 1, enalbe timer1 interrupt */
                                            /* Bit 1   = 1, clock source, PCLK */
					    /* Bit 0   = 1, enable timer1 */
#define TIMER1_INTS_CLEAR_MASK  0xfffffff8  /* Clear mask of timer status register */

#define PMUC_BASE               0x98100000  /* Base address of PMU */
#define FSSR_OFFSET		0x04	    /* Frequency Scaling Status Register */
#define PWRMODE_OFFSET          0x0C        /* Power Mode Register */
#define PLLDLLCR0_OFFSET        0x30        /* PLL/DLL Control Register 0 */

#define PLL1NS_FIELD_OFF        3           /* PLL1NS field offset */
#define PLL1NS_MASK             0x00000ff8  /* PLL1NS field mask */
#define DIV_FACTOR_FIELD_OFF    4           /* Clock dividor factor field offset */
#define DIV_FACTOR_MASK         0x000000f0  /* Clock dividor factor mask */

/************************************
 *  TARGET INTERRUPT CONSTANTS      *
 ************************************/

#define INTC_BASE              0x98800000  /* Interrupt Controller base address */
#define IRQSRCR_OFFSET         0x00        /* IRQ(HW1) Source Register offset */
#define IRQER_OFFSET           0x04        /* IRQ(HW1) Enable Register offset */
#define IRQICR_OFFSET          0x08        /* IRQ(HW1) Interrupt Clear Register offset */
#define IRQTMR_OFFSET          0x0C        /* IRQ(HW1) Trigger Mode Register offset */
#define IRQTLR_OFFSET          0x10        /* IRQ(HW1) Trigger Level Register offset */
#define IRQSR_OFFSET           0x14        /* IRQ(HW1) Status Register offset */
#define FIQSRCR_OFFSET         0x20        /* FIQ(HW0) Source Register offset */
#define FIQER_OFFSET           0x24        /* FIQ(HW0) Enable Register offset */
#define FIQICR_OFFSET          0x28        /* FIQ(HW0) Interrupt Clear Register offset */
#define FIQTMR_OFFSET          0x2C        /* FIQ(HW0) Trigger Mode Register offset */
#define FIQTLR_OFFSET          0x30        /* FIQ(HW0) Trigger Level Register offset */
#define FIQSR_OFFSET           0x34        /* FIQ(HW0) Status Register offset */
#define INT_IRQER_INIT         0x00000000  /* values used to disable all interrupts */
#define INT_CLEAR_VALUE        0xFFFFFFFF  /* values used to clear all pending interrupts */

/* Constants utilized by interrupt handler for nesting interrupts */

#define FIRST_STACK_OFFSET     40          /* First stack offset FIQ/IRQ interrupt unnset */
#define IRQER_STACK_OFFSET     40          /* INT MASK register */
#define FIQER_STACK_OFFSET     44          /* Interrupt ER OFFSET register value */
#define MASK1_STACK_OFFSET     40          /* INT MASK register */
#define IEROFF_STACK_OFFSET    44          /* Interrupt ER OFFSET register value */
#define NUM_MASK_REGS          2           /* Number of mask register (1 mask register */
                                           /* and base address of interrupt */
                                           /* control register for core module) */
#define MASK_REG_SPACE         (NUM_MASK_REGS * 4)

/************************************
 *  TARGET SDRAM CONSTANTS          *
 ************************************/

#define SDRAMC_BASE            0x90300000  /* SDRAM Controller base address */
#define PARM0_OFFSET           0x00        /* Parameter 0 Register offset */
#define PARM1_OFFSET           0x04        /* Parameter 1 Register offset */
#define CONFIG_OFFSET          0x08        /* Configuration Register offset */
#define BANK0_OFFSET           0x0C        /* Bank 0 Register offset */
#define BANK1_OFFSET           0x10        /* Bank 1 Register offset */
#define BANK2_OFFSET           0x14        /* Bank 2 Register offset */
#define BANK3_OFFSET           0x18        /* Bank 3 Register offset */
#define ARB_TIMEROUT_OFFSET    0x34        /* Arbiter Timeout Cycle Register offset */

#define SDRAMC_PARM0_INIT      0x00211312  /* SDRAMC Parameter 0 init value */
#define SDRAMC_PARM1_INIT      0x00480180  /* SDRAMC Parameter 1 init value */
#define SDRAMC_BANK0_INIT      0x10002326  /* SDRAMC bank 0 init value */
#define SDRAMC_PRECHARGE_BIT   0x00000010  /* SDRAM Precharge Action Bit for Configuration Register */
#define SDRAMC_MODE_BIT        0x00000004  /* SDRAM Mode Action Bit for Configuration Register */
#define SDRAMC_REFRESH_BIT     0x00000004  /* SDRAM Refresh Action Bit for Configuration Register */

/************************************
 *  TARGET DECODER CONSTANTS        *
 ************************************/

#define AHBC_BASE              0x90100000  /* AHB Controller base address */
#define DEVICE6_OFFSET         0x18        /* Device 6 Control Register offset */
#define INTC_OFFSET            0x88        /* Interrupt Control Register offset */

#define AHBC_DEV6_INIT         0x10080000  /* AHBC Device 6 init value */
#define AHBC_REMAP_BIT         0x00000001  /* AHBC remap bit for Interrupt Control Register */

#endif  /* AG101_REGS */
