/*
** FILE
** sio.c
**
** DESCRIPTION
** Local standard string formatting function.
**
** NOTE:
** provide psprintf, an analog of sprintf.
*/

#include <stdarg.h>
#include "sio.h"
//#include "bsp.h"
#define PRINT_OUT


//----------------------------------------------
//   BUFFER DEFINED FOR PRINT_OUT
//----------------------------------------------
#ifdef PRINT_OUT
	unsigned char linebuf[100];
#endif
//----------------------------------------------
//   BUFFER DEFINED FOR DELAY_PRINT_OUT
//----------------------------------------------
#ifdef DELAY_PRINT_OUT
	/* DBG_OUT_BUFF_LEN Should be 2^n. */
	#define  DBG_OUT_BUFF_LEN  1024 
	#define DBG_CMD_BUFF_LEN 0x40
	BYTE	DbgOutBuff[DBG_OUT_BUFF_LEN];
	BYTE	DbgCmdBuff[DBG_CMD_BUFF_LEN],DbgCmdPtr=0;
	UINT16	DbgWPtr=0;
	UINT16	DbgRPtr=0;
	BYTE	HasVfdXfer=0;
	#define MAX_TOKEN_NUM 18
#endif //DELAY_PRINT_OUT

/*
** TABLE
** radix_table
**
** DESCRIPTION
** for fast radix transform.
*/
const	char	radix_table[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

#define	utoa(dest, num, base)	__ltoa__(dest, num, base, 0)
#define	dtoa(dest, num, base)	__ltoa__(dest, num, base, CONVERT_SIGN)


/*
** conversions
*/
#define	CONVERT_SIGN		(1 << 0)
#define	CONVERT_SIGN_FORCED	(1 << 1)
#define	CONVERT_LONG		(1 << 2)
#define	CONVERT_LONGLONG	(1 << 3)
#define	CONVERT_LEADING_ZERO	(1 << 4)
#define	CONVERT_LEFT_ADJUST	(1 << 5)
#define	CONVERT_START		(1 << 15)

/*
** Some definitions to help.
*/
#define	INT16	short
#define	INT32	long
#define	INT64	long long
#define	UINT16	unsigned short
#define	UINT32	unsigned long
#define	UINT64	unsigned long long


typedef	union
{
  INT64    i64;
  INT32    i32;
  UINT64   u64;
  UINT32   u32;
  char	   *p;
} t_num;


/*
** FUNCTION
** xtoa(unsigned long x, char *s, int n)
**
** DESCRIPTION
*/
void	xtoa(unsigned long x, char *s, int n)
{
  const char *p = radix_table;
  n *= 4;
  while (n>0)
  {
    n -= 4;
    *s++ = p[(x>>n)&0x0f];
  }
  *s = '\0';
}


/*
** FUNCTION
** __ltoa16__
** 
** DESCRIPTION
** Transform a number into base-16 string.
*/
static	char	*__ltoa16__(char *dest, t_num *X, unsigned conversion, int width)
{
  char		*src, *src2;
  UINT64	num;

  /* check long */
  if (conversion&CONVERT_LONGLONG)
    num = X->u64;
  else
    num = X->u32;
    
  /* generating bytes. */
  src = dest;				/* save first pos	*/
  do {
    int frac;
    frac     = (int)(num&0x0f);
    num      = num>>4;
    src2     = dest;
    *dest++  = radix_table [ frac ];
    width--;
  } while (num);

  /* Now fill leadings (or trailing) ZEROs or SPACEs */
  {
    char c = ((conversion&(CONVERT_LEADING_ZERO|CONVERT_LEFT_ADJUST))==CONVERT_LEADING_ZERO) ? '0' : ' ';
    while (width>0)
    {
      if ((conversion&CONVERT_LEFT_ADJUST)==0)
        src2 = dest;
      *dest++ = c;
      width--;
    }
  }

  /* reverse bytes within (src src2) */
  while (src<src2)
  {
    char c;
    c     = *src2;
    *src2 = *src;
    *src  = c;
    src++;
    src2--;
  }
  return dest;
}


/*
** FUNCTION
** __ltoa__
**
** DESCRIPTION
** transform a number in base specified
*/
static	char	*__ltoa__(char *dest, t_num *X, unsigned base, int conversion, int width)
{
  char	*src, *src2;

  /* check long */
  if (conversion&CONVERT_LONGLONG)
  {
    UINT64	num = X->u64;
    /* conversion sign */
    if (conversion&CONVERT_SIGN)
    {
      if (((INT64)num)<0)
      {
        num = (UINT64)(-(INT64)num);
        *dest++ = '-';
	width--;
      }
    }  
    /* generating bytes. */
    src = dest;
    do {
      int frac;
      frac     = num%base;
      num      = num/base;
      src2     = dest;
      *dest++  = radix_table [ frac ];
      width--;
    } while (num);
  }
  else
  {
    UINT32	num = X->u32;
    /* conversion sign */
    if (conversion&CONVERT_SIGN)
    {
      if (((INT32)num)<0)
      {
        num = (UINT32)(-(INT32)num);
        *dest++ = '-';
        width--;
      }
    }  
    /* generating bytes. */
    src = dest;
    do {
      int frac;
      frac     = num%base;
      num      = num/base;
      src2     = dest;
      *dest++  = radix_table [ frac ];
      width--;
    } while (num);
  }

  /* Now fill leadings (or trailing) ZEROs or SPACEs */
  {
    char c = ((conversion&(CONVERT_LEADING_ZERO|CONVERT_LEFT_ADJUST))==CONVERT_LEADING_ZERO) ? '0' : ' ';
    while (width>0)
    {
      if ((conversion&CONVERT_LEFT_ADJUST)==0)
        src2 = dest;
      *dest++ = c;
      width--;
    }
  }

  /* reverse bytes within (src src2) */
  while (src<src2)
  {
    char c;
    c     = *src2;
    *src2 = *src;
    *src  = c;
    src++;
    src2--;
  }
  return dest;
}


int sio_vprintf(char *dest, const char *format,void* ap)
{	int		width		= 0;
	unsigned	conversion	= 0;
	while (1)
	{
		int c = *format++;
		*dest = c;
		if (c=='\0') break;
		if (conversion & CONVERT_START)
		{
			if (c>='0' && c<='9')
			{
				width = width*10 + (c-'0');
				if (width==0)
				conversion |= CONVERT_LEADING_ZERO;
			}
			else if (c=='l')
			{
				if (conversion & CONVERT_LONG) conversion |= CONVERT_LONGLONG;
				conversion |= CONVERT_LONG;
			}
			else if (c=='+')
			{
				conversion |= CONVERT_SIGN_FORCED;
			}
			else if (c=='-')
			{
				conversion = (conversion & (~CONVERT_LEADING_ZERO)) | CONVERT_LEFT_ADJUST;
			}
			else if (c=='%')
			{
				*dest++ = '%';
				conversion = 0;
			}
			else
			{
				t_num	X;
				if (conversion&CONVERT_LONGLONG)
					X.u64 = va_arg((va_list)ap,UINT64);
				else
					X.u32 = va_arg((va_list)ap,UINT32);
				switch(c)
				{
					case 'd':	dest = __ltoa__(dest,&X,10,conversion|CONVERT_SIGN,width); break;
					case 'u':	dest = __ltoa__(dest,&X,10,conversion,width); break;
					case 'x':	dest = __ltoa16__(dest,&X,conversion,width); break;
					case 's':
						{
							char *src = (char *)X.u32;
							while (*src)
							*dest++ = *src++;
						}
						break;
					case 'c':	*dest++ = (char) X.u32; break;
				}
				/* conversion finished */
				conversion = 0;
			}
		}
		else if (c=='%')
		{
			conversion	= CONVERT_START;
			width		= 0;
		}
		else
		{
			dest++;
		}
	}
	return 0;
}
/*
** FUNCTION
** psprintf
**
** DESCRIPTION
** emulation of sprintf function
*/
int	sio_psprintf(char *dest, const char *format, ...)
{
	va_list	ap;
	va_start(ap, format);
	sio_vprintf(dest,format,ap);
	va_end(ap); 
	return 0;
}



#ifdef PRINT_OUT
//----------------------------------------------
// void putc(unsigned char c)
//	put a character to VFD debuger
//		Arguments:
//			unsigned char c -- char to put
//		Return Value:
//			NONE
//----------------------------------------------
void sio_putc(register unsigned char c)
{
	bsp_outbyte(c);
	if (c==10)
	   bsp_outbyte((char) 13);
}

//----------------------------------------------
// puts(BYTE *dString)
//	put a string to VFD debuger
//		Arguments:
//			BYTE* c -- char to put
//		Return Value:
//			NONE
//----------------------------------------------
void sio_puts(register unsigned char* str)
{	while(*str!='\0')
	{	sio_putc(*str++);
	}
}
//----------------------------------------------
// void printf(const char *format, ...)
//	implement standard printf func on debuger
//		Arguments:
//			BYTE* c -- char to put
//		Return Value:
//			NONE
//----------------------------------------------
void sio_printf(const char *format, ...)
{
	va_list	ap;
	va_start(ap, format);
	sio_vprintf(linebuf,format,ap);
	va_end(ap);
	sio_puts(linebuf);
}

#endif //end of PRINT_OUT

